#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Filename:  exaile.py

import dbus
from dbus.mainloop.glib import DBusGMainLoop
from playerBase import Status
from dbusplayer import dbusPlayer

class exaile(dbusPlayer):
	
	def __init__(self):
		self.process = "exaile"
		DBusGMainLoop(set_as_default=True)
		bus_names = dbus.SessionBus().list_names()
		if "org.exaile.DBusInterface" in bus_names:
			#Exaile version 0.2
			dbusPlayer.__init__(self,"exaile","org.exaile.DBusInterface","/DBusInterfaceObject","org.exaile.DBusInterface")
			self.version = 2
		elif "org.mpris.exaile" in bus_names:
			#Exaile version 0.3 + mpris
			dbusPlayer.__init__(self,"exaile",("org.mpris.exaile","/Player","org.freedesktop.MediaPlayer"),("org.exaile.ExaileInterface","/org/exaile","org.exaile.ExaileInterface"))
			self.version = 3
			self.mpris = True
			self.status = self.checkStatus()
			#print self.status
		elif "org.exaile.ExaileInterface" in bus_names:
			#Exaile version 0.3 without mpris
			self.version = 3
			self.mpris = False
			dbusPlayer.__init__(self,"exaile","org.exaile.ExaileInterface","/org/exaile","org.exaile.ExaileInterface")

		if self.version == 2 or not self.mpris:
			#I'll leave these signal here, but seems that they don't work at all
			self.iface.connect_to_signal("playingUriChanged", self.trackChanged)
			self.iface.connect_to_signal("state_changed", self.statusChanged)
			try:
				#If installed LibNotify Plugin
				self.iface.proxy_object._bus.add_signal_receiver(
					self.trackChanged,
					signal_name="NotificationClosed",
					dbus_interface="org.freedesktop.Notifications")
			except:
				pass
		else:
			self.iface[0].connect_to_signal("StatusChange", self.statusChanged)
	
	def checkStatus(self):
		time_a = self.iface[0].PositionGet()
		#Waste some ms, just to wait
		for i in range(1024*16):
			i += 1
		time_b = self.iface[0].PositionGet()
		if time_a == time_b == 0:
			return Status().Stopped
		elif time_a == time_b:
			return Status().Paused
		else:
			return Status().Playing
	
	#TRACK SECTION
	def getTitle(self):
		if self.version == 2:
			return self.iface.get_title()
		elif self.mpris:
			tit = self.iface[1].get_track_attr("title")
			if tit.startswith("[u'") and tit.endswith("']"):
				tit = tit[3:-2]
			return tit
		else:
			return self.iface.get_track_attr("title")
	
	def getArtist(self):
		if self.version == 2:
			return self.iface.get_artist()
		elif self.mpris:
			art = self.iface[1].get_track_attr("artist")
			if art.startswith("[u'") and art.endswith("']"):
				art = art[3:-2]
			return art
		else:
			return self.iface.get_track_attr("artist")
	
	def getTimePoint(self):
		try:
			if self.version == 2:
				duration = [int(i) for i in self.iface.get_length().split(':')]
				duration = sum([duration[1-i]*60**i for i in range(2)])
				self.threshold = (duration * 10)
				percent = int(self.iface.current_position())
				time = duration * percent * 10
				return self.handleTime(time)
			elif self.mpris:
				return self.iface[0].PositionGet()
			else:
				#Actually doesn't work
				return self.iface.current_position()
		except ValueError:
			return 0
	
	def getMediaFile(self):
		if self.version == 2 or not self.mpris:
			return self.iface.get_track_attr('loc')
		else:
			return self.iface[1].get_track_attr('loc')
	
	#PLAYER SECTION
	def getStatus(self):
		if self.version == 2:
			return self.toStatus(self.iface.status())
		elif self.mpris:
			if self.status == Status().Playing:
				self.status = self.checkStatus()
			return self.status
		else:
			#doesn't have a getStatus function
			return Status().Stopped
		
	def playFunc(self):
		if self.version == 3 and self.mpris:
			self.iface[0].Play()
		else:
			self.iface.play_pause()
	
	def pauseFunc(self):
		if self.version == 3 and self.mpris:
			self.iface[0].Pause()
		else:
			self.iface.play_pause()
	
	def stopFunc(self):
		if self.version == 3 and self.mpris:
			self.iface[0].Stop()
		else:
			self.iface.stop()
	
	def prevFunc(self):
		if self.version == 3 and self.mpris:
			self.iface[0].Prev()
		else:
			self.iface.prev_track()
	
	def nextFunc(self):
		if self.version == 3 and self.mpris:
			self.iface[0].Next()
		else:
			self.iface.next_track()
	
	#EVENTS SECTION
	def toStatus(self,status):
		if status in ('playing','stoped','paused'):
			return {'playing':Status().Playing,'paused':Status().Paused,'stoped':Status().Stopped}[status]
		elif status in range(3):
			return {0:Status().Playing,1:Status().Paused,2:Status().Stopped}[status]
	
	def statusChanged(self,*arg):
		if self.callback != None:
			if arg[0] == 1:
				self.callback(Status().Paused)
			else:
				self.callback(Status().Playing)
	
	def trackChanged(self,*arg):
		if self.callback != None:
			self.callback(self.getStatus())

