#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Filename: playerBase.py

from time import time as instanttime

class Status(dict):
	def __init__(self):
		dict.__init__(self,(("Playing",0),("Paused",1),("Stopped",2)))
	
	def __getattr__(self,name):
		try:
			dict.__getattribute__(self,name)
		except Exception, e:
			try:
				return self[name]
			except:
				raise e
	

class playerBase:
	
	def __init__(self,player):
		self.callback = None
		self.song_start_time = 0
		self.threshold = 1000 # 1 sec
		if not self.isActive():
			raise Exception("Player '%s' is not running." % player)
		
	#returns True if the player is running, otherwise returns False
	def isActive(self):
		raise NotImplementedError("This class must be extended")

	#TRACK INFO SECTION
	def getTitle(self):
		raise NotImplementedError("This class must be extended")
	
	def getArtist(self):
		raise NotImplementedError("This class must be extended")
		
	def getTimePoint(self):
		raise NotImplementedError("This class must be extended")
	
	def handleTime(self,time):
		#Useful for detect exact position, not only seconds
		now = instanttime()
		theoric = (now - self.song_start_time) * 1000
		if time > theoric or abs(time - theoric) > self.threshold:
			self.song_start_time = now - time/1000.0
			return time
		else:
			return int(theoric)
	
	def getMediaFile(self):
		raise NotImplementedError("This class must be extended")
	
	def getStatus(self):
		raise NotImplementedError("This class must be extended")
		
	#PLAYER FUNCTIONS SECTION
	def getStatus(self):
		raise NotImplementedError("This class must be extended")
	
	def playFunc(self):
		raise NotImplementedError("This class must be extended")
	
	def pauseFunc(self):
		raise NotImplementedError("This class must be extended")
	
	def stopFunc(self):
		raise NotImplementedError("This class must be extended")
	
	def prevFunc(self):
		raise NotImplementedError("This class must be extended")
	
	def nextFunc(self):
		raise NotImplementedError("This class must be extended")
	
	#EVENTS SECTION:
	def onStatusChange(self,callback):
		# the callback function MUST receive a Status value (Status.Playing, Status.Stopped, Status.Paused)
		self.callback = callback

	def importChardet(self):
		if "chardet" in sys.modules:
			return sys.modules["chardet"]
		try:
			import chardet
		except:
			#get the file path
			ap = os.path.abspath(sys.modules[self.__module__].__file__)
			#get the file directory
			ap = os.path.dirname(ap)
			#get lrcShow directory
			ap = os.path.dirname(ap)
			sys.path.append(ap)
			import chardet
			sys.path.remove(ap)
		return sys.modules["chardet"]

	def autoDecode(self,string):
		try:
			chardet = self.importChardet()
			enc = chardet.detect(string)['encoding']
			if enc == "GB2312":
				try:
					string = string.decode("gb18030")
				except:
					string = string.decode("GB2312")
			elif enc == "windows-1251":
				try:
					string = string.decode("windows-1252")
				except:
					string = string.decode("windows-1251")
			elif enc != None:
				string = string.decode(enc)
			else:
				raise UnicodeDecodeError
		except:
			for enc in ('utf-8', 'cp1252', 'latin_1', 'iso8859_15', 'cp1250', 'cp1251', 'gb2312', 'big5', 'big5hkscs', 'cp037', 'cp424', 'cp437', 'cp500', 'cp737', 'cp775', 'cp850', 'cp852', 'cp855', 'cp856', 'cp857', 'cp860', 'cp861', 'cp862', 'cp863', 'cp864', 'cp865', 'cp866', 'cp869', 'cp874', 'cp875', 'cp932', 'cp949', 'cp950', 'cp1006', 'cp1026', 'cp1140', 'cp1253', 'cp1254', 'cp1255', 'cp1256', 'cp1257', 'cp1258', 'euc_jp', 'euc_jis_2004', 'euc_jisx0213', 'euc_kr', 'gbk', 'gb18030', 'hz', 'iso2022_jp', 'iso2022_jp_1', 'iso2022_jp_2', 'iso2022_jp_2004', 'iso2022_jp_3', 'iso2022_jp_ext', 'iso2022_kr', 'iso8859_2', 'iso8859_3', 'iso8859_4', 'iso8859_5', 'iso8859_6', 'iso8859_7', 'iso8859_8', 'iso8859_9', 'iso8859_10', 'iso8859_13', 'iso8859_14', 'johab', 'koi8_r', 'koi8_u', 'mac_cyrillic', 'mac_greek', 'mac_iceland', 'mac_latin2', 'mac_roman', 'mac_turkish', 'ptcp154', 'shift_jis', 'shift_jis_2004', 'shift_jisx0213', 'utf_32', 'utf_32_be', 'utf_32_le', 'utf_16', 'utf_16_be', 'utf_16_le', 'utf_7', 'utf_8_sig'):
				try:
					string = string.decode(enc)
					return string
				except:
					pass
		return string

