#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Filename: preferencePath.py

from PyQt4.QtGui import *
from PyQt4.QtCore import *
import os


class preferencePath(QWidget):
	
	def __init__(self, parent = None, *args):
		QWidget.__init__(self, parent)
		
		mainLayout = QVBoxLayout(None)
		mainLayout.setContentsMargins(5, 10, 0, 0)
		
		layout2 = QHBoxLayout(None)
		self.table = QTableWidget(0,2,self)
		self.table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        	self.table.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.table.setSelectionMode(QAbstractItemView.SingleSelection)
		self.table.setMinimumWidth(350)
		title = QStringList()<<_('Search path')<<_('Recursion depth')
		self.table.setHorizontalHeaderLabels(title)
		self.table.setColumnWidth(0,220)
		self.table.setColumnWidth(1,120)
		self.rowNumber = 0
		
		layout1 = QVBoxLayout(None)
		layout1.setContentsMargins(5, 0, 0, 0)
		self.add = QPushButton(_('Add...'), self)
		self.remove = QPushButton(_('Remove'), self)
		self.remove.setEnabled(False)
		self.up = QPushButton(_('Up'),self)
		self.up.setEnabled(False)
		self.down = QPushButton(_('Down'), self)
		self.down.setEnabled(False)
		layout1.addWidget(self.add)
		layout1.addWidget(self.remove)
		layout1.addWidget(self.up)
		layout1.addWidget(self.down)
		
		layout2.addWidget(self.table)
		layout2.addLayout(layout1)
		
		layout4 = QHBoxLayout(None)
		layout4.setContentsMargins(0,15,0,15)
		self.savePath = QLineEdit(self)
		self.savePath.setReadOnly(True)
		self.addSavePath = QPushButton(self)
		self.addSavePath.setText(_('Save path...'))
		layout4.addWidget(self.savePath)
		layout4.addWidget(self.addSavePath)
		
		self.saveToMediaPath = QCheckBox(_('Save the lrc file in track\'s path'),self)
		self.searchMediaPath = QCheckBox(_('Search the lrc file in track\'s path additionally'),self)
		
		mainLayout.addLayout(layout2)
		mainLayout.addLayout(layout4)
		mainLayout.addWidget(self.saveToMediaPath)
		mainLayout.addWidget(self.searchMediaPath)
	
		self.setLayout(mainLayout)
		
		self.connect(self.saveToMediaPath, SIGNAL("clicked(bool)"), self.cautionAction)
		self.connect(self.add, SIGNAL("clicked()"), self.addPath)
		self.connect(self.remove, SIGNAL("clicked()"), self.removePath)
		self.connect(self.up, SIGNAL("clicked()"), self.upAction)
		self.connect(self.down, SIGNAL("clicked()"), self.downAction)
		self.connect(self.table, SIGNAL("cellClicked(int,int)"), self.enableButton)
		self.connect(self.addSavePath, SIGNAL("clicked()"), self.addSavePathAction)
		self.connect(self.table, SIGNAL("cellDoubleClicked(int,int)"), self.quickAddSavePathAction)
	
	def cautionAction(self,bool):
		if(bool):
			self.addSavePath.setEnabled(False)
			self.savePath.setEnabled(False)
			a = QMessageBox(QMessageBox.Information,  _('Notice'), _('It is not recommended :\nIt would cause some players to update the collection frequently'))
			okButton = QPushButton(QIcon('./icon/ok.png'),  _('Ok'))
			a.addButton(okButton, QMessageBox.YesRole)
			a.exec_()
		else:
			self.addSavePath.setEnabled(True)
			self.savePath.setEnabled(True)
	
	def addPath(self):
		path = QFileDialog.getExistingDirectory(self, _('Please choose search lrc path'), os.getenv("HOME"))
		if(path != u''):
			flag = False
			for i in range(0, self.rowNumber):
				if(self.table.item(i, 0).text() == path):
					flag = True
			if(flag == False):
				self.insertPath(path, '0')
				self.updownButton(self.table.currentRow())
			else:
				a = QMessageBox(QMessageBox.Information,  _('Notice'), _('This path already has been added'))
				okButton = QPushButton(QIcon('./icon/ok.png'),  _('Ok'))
				a.addButton(okButton, QMessageBox.YesRole)
				a.exec_()

	def insertPath(self, path, recursion):
		self.rowNumber += 1
		self.table.setRowCount(self.rowNumber)
		self.table.setItem(self.rowNumber-1, 0, QTableWidgetItem(path, 0))
		self.table.setCellWidget(self.rowNumber-1, 1, depthComboBox(recursion))
	
	def removePath(self):
		self.table.removeRow(self.table.currentRow())
		self.rowNumber -= 1
		self.updownButton(self.table.currentRow())
	
	def enableButton(self, row, column):
		self.remove.setEnabled(True)
		self.updownButton(row)
	
	def upAction(self):
		row = self.table.currentRow()
		item_column0 = self.table.takeItem(row, 0)
		item_column1 = self.table.cellWidget(row, 1).currentText()
		tmp_row = row-1
		tmp_column0 = self.table.takeItem(tmp_row, 0)
		tmp_column1 = self.table.cellWidget(tmp_row, 1).currentText()
		self.table.setItem(tmp_row, 0, item_column0)
		self.table.setCellWidget(tmp_row, 1, depthComboBox(item_column1))
		self.table.setItem(row, 0, tmp_column0)
		self.table.setCellWidget(row, 1, depthComboBox(tmp_column1))
		
		self.table.selectRow(tmp_row)
		self.updownButton(tmp_row)
	
	def downAction(self):
		row = self.table.currentRow()
		item_column0 = self.table.takeItem(row, 0)
		item_column1 = self.table.cellWidget(row, 1).currentText()
		tmp_row = row+1
		tmp_column0 = self.table.takeItem(tmp_row, 0)
		tmp_column1 = self.table.cellWidget(tmp_row, 1).currentText()
		self.table.setItem(row, 0, tmp_column0)
		self.table.setCellWidget(row, 1, depthComboBox(tmp_column1))
		self.table.setItem(tmp_row, 0, item_column0)
		self.table.setCellWidget(tmp_row, 1, depthComboBox(item_column1))
		self.table.selectRow(tmp_row)
		self.updownButton(tmp_row)
	
	def addSavePathAction(self):
		path = QFileDialog.getExistingDirectory(self, _('Please choose a path for saving lrc'), os.getenv("HOME"))
		if(path != u''):
			if(os.access(str(path.toLocal8Bit()), os.W_OK)):
				self.savePath.setText(path)
			else:
				a = QMessageBox(QMessageBox.Information,  _('Notice'), _('It does not have writing permission'))
				okButton = QPushButton(QIcon('./icon/ok.png'),  _('Ok'))
				a.addButton(okButton, QMessageBox.YesRole)
				a.exec_()
	
	def quickAddSavePathAction(self, row, col):
		url = self.table.item(row, 0).text()
		if(os.access(str(url.toLocal8Bit()), os.W_OK)):
			self.savePath.setText(url)
		
	def updownButton(self,row):
		if(row == 0):
			self.up.setEnabled(False)
		else:
			self.up.setEnabled(True)
		if(row == self.rowNumber-1):
			self.down.setEnabled(False)
		else:
			self.down.setEnabled(True)
		if(row < 0):
			self.up.setEnabled(False)
			self.down.setEnabled(False)
			self.remove.setEnabled(False)


class depthComboBox(QComboBox):
	
	def __init__(self, t, parent = None):
		QComboBox.__init__(self, parent)
		itemList = QStringList() << '0' << '1' << '2' << '3'
		self.addItems(itemList)
		self.setCurrentIndex(itemList.indexOf(t))