#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Filename: trackInfo.py
# Author: Xu Jia (Sanfanling)   E-mail:xujia19@gmail.com
# Lisence: GPL-2.0

from PyQt4.QtCore import *
from PyQt4.QtGui import *
import sys, os

class trackInfo(QDialog):
	
	def __init__(self, parent = None):
        	QDialog.__init__(self, parent)
        	self.setWindowTitle(_('Track Information'))
        	mainLayout = QVBoxLayout(None)
        	tabWidget = QTabWidget()
        	self.generalTab = generalTab()
        	self.strategyTab = strategyTab()
        	tabWidget.addTab(self.generalTab, _('General'))
        	tabWidget.addTab(self.strategyTab, _('Strategy'))

		buttonBox = QDialogButtonBox(self)
		okButton = QPushButton(QIcon('./icon/ok.png'), _('Ok'))
		cancelButton = QPushButton(QIcon('./icon/cancel.png'), _('Cancel'))
        	buttonBox.addButton(okButton, QDialogButtonBox.AcceptRole)
        	buttonBox.addButton(cancelButton, QDialogButtonBox.RejectRole)
        	
        	mainLayout.addWidget(tabWidget)
        	mainLayout.addWidget(buttonBox)
        	self.setLayout(mainLayout)
        	
        	self.connect(buttonBox, SIGNAL('accepted()'), self.checkAccept)
        	self.connect(buttonBox, SIGNAL('rejected()'), self, SLOT('reject()'))
        
        def checkAccept(self):
		if(self.strategyTab.pathLine.text() == u'' and self.strategyTab.comboBox.currentIndex() == 4):
			b = QMessageBox(self)
			b.setWindowTitle('Error')
			b.setText('Please choose a lrc file')
			b.setIcon(QMessageBox.Critical)
			b.addButton(_('Ok'), QMessageBox.AcceptRole)
			b.exec_()
		else:
			self.accept()

class generalTab(QWidget):
	
	def __init__(self, parent = None, *args):
		QWidget.__init__(self, parent)
		mainLayout = QVBoxLayout(None)
		
		layout1 = QHBoxLayout(None)
		layout1.setSpacing(15)
		fileNameLabel = QLabel(_('Audio file:'), self)
		self.fileName = QLabel(self)
		layout1.addWidget(fileNameLabel)
		layout1.addWidget(self.fileName)
		
		layout2 = QHBoxLayout(None)
		layout2.setSpacing(15)
		artistLabel = QLabel(_('Artist:'), self)
		self.artist= QLabel(self)
		layout2.addWidget(artistLabel)
		layout2.addWidget(self.artist)
		
		layout3 = QHBoxLayout(None)
		layout3.setSpacing(15)
		titleLabel = QLabel(_('Title:'), self)
		self.title = QLabel(self)
		layout3.addWidget(titleLabel)
		layout3.addWidget(self.title)
		
		layout4 = QHBoxLayout(None)
		layout4.setSpacing(15)
		lrcFileLabel = QLabel(_('Lrc file:'), self)
		self.lrcFile = QLabel(self)
		layout4.addWidget(lrcFileLabel)
		layout4.addWidget(self.lrcFile)
		
		mainLayout.addLayout(layout2)
		mainLayout.addLayout(layout3)
		mainLayout.addLayout(layout1)
		mainLayout.addLayout(layout4)
		self.setLayout(mainLayout)

class strategyTab(QWidget):
	
	def __init__(self, parent = None):
		QWidget.__init__(self, parent)
		mainLayout = QVBoxLayout(None)
		
		layout1 = QHBoxLayout(None)
		layout1.setSpacing(15)
		layout1.setContentsMargins(0,10,0,10)
		label = QLabel(_('Current strategy:'), self)
		self.comboBox = QComboBox(self)
		self.aList = [_('No strategy'), _('Do not search local'), _('Do not search web'), _('Do not search at all'), _('Remember...')]
		for i in self.aList:
			self.comboBox.insertItem(self.aList.index(i), i)
		layout1.addWidget(label)
		layout1.addWidget(self.comboBox)
		
		layout2 = QHBoxLayout(None)
		layout2.setContentsMargins(0,10,0,20)
		self.pathLine = QLineEdit(self)
		self.pathLine.setReadOnly(True)
		self.pathLine.setEnabled(False)
		self.chooseButton = QPushButton(_('Choose...'), self)
		self.chooseButton.setEnabled(False)
		layout2.addWidget(self.pathLine)
		layout2.addWidget(self.chooseButton)
		
		mainLayout.addLayout(layout1)
		mainLayout.addLayout(layout2)
		self.setLayout(mainLayout)
		
		self.connect(self.chooseButton, SIGNAL("clicked()"), self.chooseLrc)
		self.connect(self.comboBox, SIGNAL("currentIndexChanged(int)"), self.buttonStatusChange)
		
	
	def chooseLrc(self):
		if(self.pathLine.text() != u''):
			tmpDir = self.pathLine.text()
		else:
			tmpDir = os.getenv('HOME')
		path=QFileDialog.getOpenFileName(self, _('Please choose a lrc file'), tmpDir, 'Lrc file (*.lrc)')
		if(path != u''):
			self.pathLine.setText(path)
	
	def buttonStatusChange(self, i):
		if(i == 0 or i == 1 or i == 2 or i == 3):
			self.pathLine.setEnabled(False)
			self.chooseButton.setEnabled(False)
		else:
			self.pathLine.setEnabled(True)
			self.chooseButton.setEnabled(True)
	
	def setCurrentStrategy(self, strategy = None, lrcFile = None):
		if(strategy == None):
			self.comboBox.setCurrentIndex(0)
			self.pathLine.setEnabled(False)
			self.chooseButton.setEnabled(False)
		elif(strategy == 'FLAG.SKIP_SEARCHLOCAL'):
			self.comboBox.setCurrentIndex(1)
			self.pathLine.setEnabled(False)
			self.chooseButton.setEnabled(False)
		elif(strategy == 'FLAG.SKIP_SEARCHENGINE'):
			self.comboBox.setCurrentIndex(2)
			self.pathLine.setEnabled(False)
			self.chooseButton.setEnabled(False)
		elif(strategy == 'FLAG.SKIP_ALL'):
			self.comboBox.setCurrentIndex(3)
			self.pathLine.setEnabled(True)
			self.chooseButton.setEnabled(True)
			
		if(lrcFile != None):
			self.pathLine.setText(unicode(lrcFile, sys.getfilesystemencoding()))